@@ -11,7 +11,7 @@
 {-# LANGUAGE ConstraintKinds #-}
 {-# LANGUAGE ImplicitParams #-}
 
--- Note: this module is marked 'Unsafe' because it exports (#.), which is essentially 'coerce', and Data.Coerce is marked 'Unsafe' in base. As per <https://github.com/ekmett/lens/issues/661>, this is an issue for 'lens' as well but they have opted for 'Trustworthy' instead.
+-- Note: this module is marked 'Unsafe' because it exports 'coerce', and Data.Coerce is marked 'Unsafe' in base. As per <https://github.com/ekmett/lens/issues/661>, this is an issue for 'lens' as well but they have opted for 'Trustworthy' instead.
 {-# LANGUAGE Unsafe #-}
 
 {- |
@@ -36,8 +36,6 @@ module Lens.Micro.Internal
   foldrOf,
   foldMapOf,
   sets,
-  ( #. ),
-  ( .# ),
   phantom,
   Each(..),
   Index,
@@ -56,6 +54,13 @@ module Lens.Micro.Internal
 
   -- * CallStack
   HasCallStack,
+
+  -- * Coerce compatibility shim
+  coerce,
+
+  -- * Coerce-like composition
+  ( #. ),
+  ( .# ),
 )
 where
 
@@ -69,7 +74,7 @@ import Data.Functor.Identity
 import Data.Complex
 
 #if __GLASGOW_HASKELL__ >= 800
-import Data.List.NonEmpty (NonEmpty)
+import Data.List.NonEmpty (NonEmpty(..))
 #endif
 
 #if __GLASGOW_HASKELL__ < 710
@@ -153,34 +158,6 @@ noEffect :: Monoid r => Const r a
 noEffect = phantom (pure ())
 {-# INLINE noEffect #-}
 
-------------------------------------------------------------------------------
--- Data.Profunctor.Unsafe
-------------------------------------------------------------------------------
-
--- Note: 'lens' defines a type-restricted version of (#.) to work around a
--- bug, but our version is restricted enough that we don't need it. See
--- <https://github.com/ekmett/lens/commit/cde2fc39c0dba413d1a6f814b47bd14431a5e339>
-
-#if __GLASGOW_HASKELL__ >= 708
-( #. ) :: Coercible c b => (b -> c) -> (a -> b) -> (a -> c)
-( #. ) _ = coerce (\x -> x :: b) :: forall a b. Coercible b a => a -> b
-
-( .# ) :: Coercible b a => (b -> c) -> (a -> b) -> (a -> c)
-( .# ) pbc _ = coerce pbc
-#else
-( #. ) :: (b -> c) -> (a -> b) -> (a -> c)
-( #. ) _ = unsafeCoerce
-
-( .# ) :: (b -> c) -> (a -> b) -> (a -> c)
-( .# ) pbc _ = unsafeCoerce pbc
-#endif
-
-{-# INLINE ( #. ) #-}
-{-# INLINE ( .# ) #-}
-
-infixr 9 #.
-infixl 8 .#
-
 ------------------------------------------------------------------------------
 -- classes
 ------------------------------------------------------------------------------
@@ -270,6 +247,11 @@ type instance IxValue (e -> a) = a
 type instance Index   [a] = Int
 type instance IxValue [a] = a
 
+#if __GLASGOW_HASKELL__ >= 800
+type instance Index   (NonEmpty a) = Int
+type instance IxValue (NonEmpty a) = a
+#endif
+
 class Ixed m where
   {- |
 This traversal lets you access (and update) an arbitrary element in a list, array, @Map@, etc. (If you want to insert or delete elements as well, look at 'at'.)
@@ -297,11 +279,21 @@ maximum0 = 'maximum' 'Lens.Micro.&' 'ix' [] 'Lens.Micro..~' 0
 
 The following instances are provided in this package:
 
+#if __GLASGOW_HASKELL__ >= 800
+@
+'ix' :: 'Int' -> 'Traversal'' [a] a
+
+'ix' :: 'Int' -> 'Traversal'' (NonEmpty a) a
+
+'ix' :: ('Eq' e) => e -> 'Traversal'' (e -> a) a
+@
+#else
 @
 'ix' :: 'Int' -> 'Traversal'' [a] a
 
 'ix' :: ('Eq' e) => e -> 'Traversal'' (e -> a) a
 @
+#endif
 
 You can also use 'ix' with types from <http://hackage.haskell.org/package/array array>, <http://hackage.haskell.org/package/bytestring bytestring>, and <http://hackage.haskell.org/package/containers containers> by using <http://hackage.haskell.org/package/microlens-ghc microlens-ghc>, or additionally with types from <http://hackage.haskell.org/package/vector vector>, <http://hackage.haskell.org/package/text text>, and <http://hackage.haskell.org/package/unordered-containers unordered-containers> by using <http://hackage.haskell.org/package/microlens-platform microlens-platform>.
   -}
@@ -360,6 +352,15 @@ instance Ixed [a] where
     go (a:as) i = (a:) <$> (go as $! i - 1)
   {-# INLINE ix #-}
 
+#if __GLASGOW_HASKELL__ >= 800
+instance Ixed (NonEmpty a) where
+  ix k f xs0 | k < 0 = pure xs0
+             | otherwise = go xs0 k where
+    go (a:|as) 0 = (:|as) <$> f a
+    go (a:|as) i = (a:|) <$> ix (i - 1) f as
+  {-# INLINE ix #-}
+#endif
+
 class Field1 s t a b | s -> a, t -> b, s b -> t, t a -> s where
   {- |
 Gives access to the 1st field of a tuple (up to 5-tuples).
@@ -618,3 +619,41 @@ stripDiacritics = ...
 "hello" :: Lazy.Text
   -}
   lazy   :: Lens' strict lazy
+
+----------------------------------------------------------------------------
+-- Coerce compatibility shim
+----------------------------------------------------------------------------
+
+#if __GLASGOW_HASKELL__ < 708
+coerce :: a -> b
+coerce = unsafeCoerce
+{-# INLINE coerce #-}
+#endif
+
+----------------------------------------------------------------------------
+-- Coerce-like composition
+----------------------------------------------------------------------------
+
+-- Note: 'lens' defines a type-restricted version of (#.) to work around a
+-- bug, but our version is restricted enough that we don't need it. See
+-- <https://github.com/ekmett/lens/commit/cde2fc39c0dba413d1a6f814b47bd14431a5e339>
+
+#if __GLASGOW_HASKELL__ >= 708
+( #. ) :: Coercible c b => (b -> c) -> (a -> b) -> (a -> c)
+( #. ) _ = coerce (\x -> x :: b) :: forall a b. Coercible b a => a -> b
+
+( .# ) :: Coercible b a => (b -> c) -> (a -> b) -> (a -> c)
+( .# ) pbc _ = coerce pbc
+#else
+( #. ) :: (b -> c) -> (a -> b) -> (a -> c)
+( #. ) _ = unsafeCoerce
+
+( .# ) :: (b -> c) -> (a -> b) -> (a -> c)
+( .# ) pbc _ = unsafeCoerce pbc
+#endif
+
+{-# INLINE ( #. ) #-}
+{-# INLINE ( .# ) #-}
+
+infixr 9 #.
+infixl 8 .#
